@charset "UTF-8";

* {
	margin: 0;
	padding: 0;
}

html {
	height: 100%;
}

body {
	height: 100%;
	background-color: #fff;
	color: #000;
	font-family: Tahoma, sans-serif;
}

a, a:link, a:active, a:visited, a:focus {
	color: #00008B;
	text-decoration: none;
	outline: none;
}

a:hover {
	text-decoration: underline;
}

#main a:visited {
	color: #99008B;
}

#wrapper {
	background-color: #eee;
	min-height: 80%;
	overflow: hidden;
	margin: 0 auto;
	width: 900px;

	border: 1px solid #ccc;
	border-top: none;
	-moz-box-shadow: 0 0 1em #666;
	-webkit-box-shadow: 0 0 1em #666;
	box-shadow: 0 0 1em #666;
}

#banner_top {
	background-color: #ccc;
}

#banner_top a {
	color: #000;
	text-decoration: none;
}

h1 {
	line-height: 37px;
	padding: 0 0.9em 0 0.6em;
	width: 50px;
	text-align: center;
	background-color: #aaa;
	font-weight: normal;
	font-style: italic;
	font-size: 100%;
}

#tag {
	height: 4em;
	line-height: 4em;
	text-align: center;
	font-size: 70%;
	color: #aaa;
}

#main {
	padding: 0 20px 20px 20px;
	font-size: 90%;
	width: 680px;
}

#menu {
	border-left: 2px solid #aaa;
	padding-left: 10px;
	position: absolute;
	margin-left: 700px;
	width: 170px;
	font-size: 14px;
}

ul.nav, ul.nav li, ul.nav ul {
	list-style: none;
}

ul.nav li.topitem {
	margin-right: 55px;
}

ul.nav li, ul.nav li.topitem .title {
	display: block;
	margin-bottom: 5px;
}

ul.nav li.subitem {
	font-size: 90%;
}

ul.nav li.topitem:before,
ul.nav li.modtitle:before {
	content: "» ";
}

ul.nav li.topitem, ul.nav li.modtitle {
	padding-left: 3px;
}

ul.nav li.parent:before {
	content: "";
}

ul.nav li.parent {
	padding-left: 0;
}

ul.nav li.subitem {
	margin-left: 20px;
}

#infobar {
	text-align: right;
	clear: both;
	position: relative;
	top: -30px;
	margin-left: 485px;
	background-color: #ccc;
	color: #333;
	padding: 0 6px 3px 0;
}

.errorbar + #infobar {
	top: -59px;
}

#infobar input {
	border: none;
	font-family: Tahoma, sans-serif;
	padding: 1px;
	font-size: 14px;
	margin: 0 4px;
	width: 120px;
}

#infobar input.submit {
	border: 1px solid #333;
	font-size: 12px;
	width: 60px;
}

#infobar_ident, .logoutbox, .loginbox {
	padding: 4px;
	font-size: 70%;
}

#infobar_ident {
	margin-right: 4px;
}

.logoutbox, .loginbox {
	padding-top: 7px;
}

.successbar,
.errorbar {
	width: 100%;
	background-color: #900;
	padding: 5px 10px;
	font-weight: bold;
	color: #fff;
}

.successbar {
	background-color: #070;
}

h2 {
	margin-bottom: 0.4em;
}

h3 {
	margin-top: 1.2em;
	margin-bottom: 0.4em;
}

.textsection, .toptable {
	width: 670px;
	overflow: hidden;
}

.section {
	clear: both;
}

.sectionbody {
	margin-left: 1em;
}

.sectionbody input, textarea, select {
	border: 2px solid #ccc;
	font-family: Tahoma, sans-serif;
	padding: 3px;
	font-size: 14px;
}

input:focus, textarea:focus, select:focus {
	-moz-box-shadow: 0 0 0.4em #aaa;
	-webkit-box-shadow: 0 0 0.4em #aaa;
	box-shadow: 0 0 0.4em #aaa;
	outline: none; /* for webkit */
}

textarea {
	height: 90px;
}

div.submitline {
	margin: 1em;
	padding-top: 1em;
	border-top: 1px solid #aaa;
}

.submitline input {
	width: 200px;
	height: 2.2em;
}

input.full, textarea.full,
.full input, .full textarea {
	width: 500px;
}

input.twothird, textarea.twothird,
.twothird > input, .twothird > textarea {
	width: 450px;
}

input.half, textarea.half,
.half > input, .half > textarea {
	width: 300px;
}

input.third, textarea.third,
.third > input, .third > textarea {
	width: 150px;
}

input.sixth, textarea.sixth,
.sixth > input, .sixth > textarea {
	width: 75px;
}

input[type=number] {
	width: 75px;
	min-width: 75px;
}

table {
	border: 1px solid #ccc;
	border-spacing: 1px;
}

td, th {
	padding: 5px 7px;
	min-width: 35px;
}

thead td, th {
	background-color: #a0a0a0;
}

th {
	text-align: left;
}

tbody td {
	background-color: #cecece;
}

tr.evenrow td {
	background-color: #dadada;
}

.info {
	font-style: italic;
	font-size: 80%;
}

.subsection::after {
	content: "";
	clear: both;
	display: table;
}

.subsection div {
	float: left;
}

.subsection .inputlabel {
	width: 120px;
	text-align: right;
	padding: 10px 5px 0 0;
}

.subsection > input, .subsection select, .subsection textarea {
	margin: 5px 0 5px 0;
	min-width: 75px;
	vertical-align: middle;
}

.subsection div.checkbox {
	padding: 9px 0 0 3px;
	max-width: 537px;
}

.subsection div.checkbox label {
	vertical-align: top;
	display: inline-block;
	max-width:510px;
}

.section .info {
	margin-bottom: 5px;
	display: block;
}

.subsection .info {
	text-align: right;
}

.half .info {
	width: 435px;
}

.third .info {
	width: 285px;
}

.twothird .info {
	width: 585px;
}

.full .info {
	width: 610px;
	clear: both;
}

td.mod_descr,
td.mod_name,
td.mod_args input {
	font-size: 80%;
}

td.mod_name,
.checkboxandlabel {
	white-space: nowrap;
}

td.center {
	text-align: center;
}

.lotsofcheckboxes .checkboxandlabel {
	display: block;
	float: left;
	width: 100%;
	margin-top: 0.5em;
}

#breadcrumb {
	width: 670px;
	padding: 0 0 3px 1px;
	margin-bottom: 10px;
	border-bottom: 1px solid #aaa;
}

#servers_js > div:first-child {
	font-weight: bold;
	margin: 20px 0 8px;
}

td {
	word-wrap: break-word;
}

.textsection p {
	margin-bottom: 0.7em;
}

input.channel_index {
  width: 3em;
  min-width: unset;
}
